@@ -13,7 +13,7 @@ class BaseModel(pydantic.BaseModel, allow_mutation=False):
 
     def __repr_args__(self: pydantic.BaseModel):
         return [
-            (key, value) for key, value in self.__dict__.items() if value is not self.__fields__[key].field_info.default
+            (key, value) for key, value in self.__dict__.items() if value != self.__fields__[key].field_info.default
         ]
 
     def update(self, **kwargs):
