@@ -7,14 +7,15 @@ import logging
 import shutil
 import urllib
 from pathlib import Path
-from typing import Generator, List, Optional
+from typing import Generator, List, Optional, Tuple
 
-from ._git import Git
 from ._util import no_echo, removesuffix, resolve_relative, run
+from .clone import Clone, map_paths
 from .const import MANIFEST_PATH_DEFAULT
 from .datamodel import Manifest, ManifestSpec, Project, ProjectSpec
-from .exceptions import GitCloneMissingError, ManifestExistError
+from .exceptions import GitCloneMissingError, GitCloneNotCleanError, ManifestExistError
 from .filters import Filter, default_filter
+from .git import Git, Status
 from .iters import ManifestIter, ProjectIter
 from .types import Groups, ProjectFilter
 from .workspace import Workspace
@@ -34,6 +35,8 @@ class AnyRepo:
         manifest_spec (ManifestSpec): manifest.
     """
 
+    # pylint: disable=too-many-public-methods
+
     def __init__(self, workspace: Workspace, manifest_spec: ManifestSpec, echo=None):
         self.workspace = workspace
         self.manifest_spec = manifest_spec
@@ -97,11 +100,15 @@ class AnyRepo:
         echo = echo or no_echo
         project_path = Git.find_path(path=project_path)
         name = project_path.name
-        echo(f"===== {name} (revision=None, path={name!r}) =====", fg=_COLOR_BANNER)
+        echo(f"===== {name} =====", fg=_COLOR_BANNER)
         manifest_path = resolve_relative(project_path / manifest_path)
         path = project_path.parent
         return AnyRepo.create(path, project_path, manifest_path, groups, echo=echo)
 
+    def deinit(self):
+        """De-Initialize :any:`AnyRepo`."""
+        return self.workspace.deinit()
+
     @staticmethod
     def clone(
         url: str,
@@ -115,7 +122,7 @@ class AnyRepo:
         path = path or Path.cwd()
         parsedurl = urllib.parse.urlparse(url)
         name = Path(parsedurl.path).name
-        echo(f"===== {name} (revision=None, path={name!r}) =====", fg=_COLOR_BANNER)
+        echo(f"===== {name} =====", fg=_COLOR_BANNER)
         echo(f"Cloning {url!r}.", fg=_COLOR_ACTION)
         project_path = path / removesuffix(name, ".git")
         git = Git(project_path)
@@ -130,41 +137,43 @@ class AnyRepo:
         skip_main: bool = False,
         prune: bool = False,
         rebase: bool = False,
+        force: bool = False,
     ):
         """Create/Update all dependent projects."""
         workspace = self.workspace
         used: List[Path] = [workspace.info.main_path]
-        for project in self.iter(
+        for clone in self.foreach(
             project_paths=project_paths,
             manifest_path=manifest_path,
             groups=groups,
             skip_main=skip_main,
             resolve_url=True,
+            no_warn=True,
         ):
-            used.append(Path(project.path))
-            project_path = workspace.get_project_path(project, relative=True)
-            git = Git(project_path)
-            self._update(git, project, rebase)
+            used.append(Path(clone.project.path))
+            self._update(clone, rebase)
         if prune:
-            self._prune(workspace, used)
+            self._prune(workspace, used, force=force)
 
-    def _update(self, git, project, rebase):
+    def _update(self, clone: Clone, rebase: bool):
         # Clone
+        project = clone.project
+        git = clone.git
         if not git.is_cloned():
             self.echo(f"Cloning {project.url!r}.", fg=_COLOR_ACTION)
             git.clone(project.url, revision=project.revision)
             return
 
         # Determine actual version
-        sha = git.get_sha()
         tag = git.get_tag()
         branch = git.get_branch()
+        sha = git.get_sha()
 
         if project.revision in (sha, tag) and not branch:
             self.echo("Nothing to do.", fg=_COLOR_ACTION)
             return
 
-        revision = branch or tag or sha
+        revision = tag or branch or sha
 
         # Checkout
         fetched = False
@@ -175,7 +184,7 @@ class AnyRepo:
             self.echo(f"Checking out {project.revision!r} (previously {revision!r}).", fg=_COLOR_ACTION)
             git.checkout(project.revision)
             branch = git.get_branch()
-            revision = branch or tag or sha
+            revision = tag or branch or sha
 
         # Pull or Rebase in case we are on a branch (or have switched to it.)
         if branch:
@@ -193,45 +202,113 @@ class AnyRepo:
                     self.echo(f"Merging branch {branch!r}.", fg=_COLOR_ACTION)
                     git.merge()
 
-    def _prune(self, workspace: Workspace, used: List[Path]):
+    def _prune(self, workspace: Workspace, used: List[Path], force: bool = False):
         for obsolete_path in workspace.iter_obsoletes(used):
             name = resolve_relative(obsolete_path, workspace.path)
             self.echo(f"===== {name} (OBSOLETE) =====", fg=_COLOR_BANNER)
             self.echo(f"Removing {str(obsolete_path)!r}.", fg=_COLOR_ACTION)
-            # TODO: safety check.
-            shutil.rmtree(obsolete_path, ignore_errors=True)
+            git = Git(obsolete_path)
+            if force or not git.is_cloned() or git.is_clean():
+                shutil.rmtree(obsolete_path, ignore_errors=True)
+            else:
+                raise GitCloneNotCleanError(resolve_relative(obsolete_path))
 
-    def foreach(self, command, project_paths=None, manifest_path: Path = None, groups: Groups = None):
+    def status(
+        self,
+        project_paths=None,
+        manifest_path: Path = None,
+        groups: Groups = None,
+    ) -> Generator[Status, None, None]:
+        """Iterate over Status."""
+        for clone in self.foreach(project_paths=project_paths, manifest_path=manifest_path, groups=groups):
+            path = clone.git.path
+            for status in clone.git.status():
+                yield status.with_path(path)
+
+    def checkout(self, paths: Tuple[Path, ...]):
+        """Checkout."""
+        if paths:
+            # Checkout specific files only
+            for clone, cpaths in map_paths(tuple(self.clones()), paths):
+                self._echo_project_banner(clone.project)
+                if cpaths:
+                    clone.git.checkout(revision=clone.project.revision, paths=cpaths)
+        else:
+            # Checkout all branches
+            for clone in self.clones():
+                self._echo_project_banner(clone.project)
+                if clone.project.revision:
+                    clone.git.checkout(revision=clone.project.revision)
+
+    def add(self, paths: Tuple[Path, ...]):
+        """Add."""
+        for clone, cpaths in map_paths(tuple(self.clones()), paths):
+            if cpaths:
+                clone.git.add(cpaths)
+
+    def reset(self, paths: Tuple[Path, ...]):
+        """Reset."""
+        for clone, cpaths in map_paths(tuple(self.clones()), paths):
+            if cpaths:
+                clone.git.reset(cpaths)
+
+    def commit(self, msg: str, paths: Tuple[Path, ...]):
+        """Commit."""
+        for clone, cpaths in map_paths(tuple(self.clones()), paths):
+            if cpaths:
+                self._echo_project_banner(clone.project)
+                clone.git.commit(msg, paths=cpaths)
+
+    def run_foreach(self, command, project_paths=None, manifest_path: Path = None, groups: Groups = None):
         """Run `command` on each project."""
-        workspace = self.workspace
-        for project in self.iter(project_paths=project_paths, manifest_path=manifest_path, groups=groups):
-            project_path = workspace.get_project_path(project, relative=True)
-            git = Git(project_path)
-            if not git.is_cloned():
-                raise GitCloneMissingError(resolve_relative(project_path))
-            run(command, cwd=project_path)
+        for clone in self.foreach(project_paths=project_paths, manifest_path=manifest_path, groups=groups):
+            if not clone.git.is_cloned():
+                raise GitCloneMissingError(clone.git.path)
+            run(command, cwd=clone.git.path)
 
-    def iter(
+    def foreach(
         self,
         project_paths=None,
         manifest_path: Path = None,
         groups: Groups = None,
         skip_main: bool = False,
         resolve_url: bool = False,
-    ) -> Generator[Project, None, None]:
-        """Iterate of all dependent projects."""
+        no_warn: bool = False,
+    ) -> Generator[Clone, None, None]:
+        """User Level Clone Iteration."""
         project_paths_filter = self._create_project_paths_filter(project_paths)
-        groups = self.workspace.get_groups(groups)
-        filter_ = self.create_groups_filter(groups)
+        groups = self.workspace.get_groups(groups=groups)
+        filter_ = self.create_groups_filter(groups=groups)
         if groups:
             self.echo(f"Groups: {groups!r}", bold=True)
-        for project in self.projects(manifest_path, filter_=filter_, skip_main=skip_main, resolve_url=resolve_url):
+        for clone in self.clones(manifest_path, filter_, skip_main=skip_main, resolve_url=resolve_url):
+            project = clone.project
+            projectrev = project.revision
+            try:
+                clonerev = clone.git.get_revision()
+            except FileNotFoundError:
+                clonerev = None
             if project_paths_filter(project):
-                self.echo(f"===== {project.info} =====", fg=_COLOR_BANNER)
-                yield project
+                self._echo_project_banner(clone.project)
+                if not no_warn and projectrev and clonerev and projectrev != clonerev:
+                    _LOGGER.warning("Clone %s is on different revision: %r", project.info, clonerev)
+                yield clone
             else:
                 self.echo(f"===== SKIPPING {project.info} =====", fg=_COLOR_SKIP)
 
+    def clones(
+        self,
+        manifest_path: Path = None,
+        filter_: ProjectFilter = None,
+        skip_main: bool = False,
+        resolve_url: bool = False,
+    ) -> Generator[Clone, None, None]:
+        """Iterate over Clones."""
+        workspace = self.workspace
+        for project in self.projects(manifest_path, filter_, skip_main=skip_main, resolve_url=resolve_url):
+            clone = Clone.from_project(workspace, project)
+            yield clone
+
     def projects(
         self,
         manifest_path: Optional[Path] = None,
@@ -242,6 +319,7 @@ class AnyRepo:
         """Iterate over Projects."""
         workspace = self.workspace
         manifest_path = workspace.get_manifest_path(manifest_path=manifest_path)
+        filter_ = filter_ or self.create_groups_filter()
         yield from ProjectIter(workspace, manifest_path, filter_=filter_, skip_main=skip_main, resolve_url=resolve_url)
 
     def manifests(
@@ -250,16 +328,12 @@ class AnyRepo:
         """Iterate over Manifests."""
         workspace = self.workspace
         manifest_path = workspace.get_manifest_path(manifest_path=manifest_path)
+        filter_ = filter_ or self.create_groups_filter()
         yield from ManifestIter(workspace, manifest_path, filter_=filter_)
 
     @staticmethod
-    def create_manifest(project_path: Path = None, manifest_path: Path = MANIFEST_PATH_DEFAULT) -> Path:
+    def create_manifest(manifest_path: Path = MANIFEST_PATH_DEFAULT) -> Path:
         """Create ManifestSpec File at `manifest_path`within `project`."""
-        if project_path is None:
-            project_path = manifest_path.parent
-            manifest_path = manifest_path.relative_to(project_path)
-        git = Git.from_path(path=project_path)
-        manifest_path = resolve_relative(git.path / manifest_path)
         if manifest_path.exists():
             raise ManifestExistError(manifest_path)
         manifest_spec = ManifestSpec()
@@ -306,8 +380,11 @@ class AnyRepo:
             return lambda project: resolve_relative(workspace_path / project.path, base=workspace_path) in project_paths
         return default_filter
 
-    def create_groups_filter(self, groups):
+    def create_groups_filter(self, groups=None):
         """Create Filter Method for `groups`."""
+        if groups is None:
+            groups = self.workspace.get_groups()
+
         filter_ = Filter.from_str(groups or "")
 
         def func(project):
@@ -316,3 +393,6 @@ class AnyRepo:
             return filter_(groups, disabled=disabled)
 
         return func
+
+    def _echo_project_banner(self, project):
+        self.echo(f"===== {project.info} =====", fg=_COLOR_BANNER)
